(function($, globals) {

	'use strict';

	var MyAMS = globals.MyAMS;

	var PyAMS_portal = {

        /**
		 * I18n strings
         */
		i18n: {
			CANT_DELETE_ROW_WITH_PORTLETS: "A row containing portlets dan't be removed!",
			CANT_DELETE_SLOT_WITH_PORTLETS: "A slot containing portlets can't be removed!"
		},

		/**
		 * Templates management
		 */
		template: {

			initConfig: function() {
				var config = $('#portal_config');
				if (config.data('ams-allowed-change')) {
					// Init sortables and resizables
					$('.rows', config).addClass('sortable');
					$('.slots', config).addClass('sortable');
					$('.slot', config).addClass('resizable');
					$('.portlets', config).addClass('sortable');
					MyAMS.plugins.enabled.sortable(config);
					MyAMS.plugins.enabled.resizable(config);
					// Init rows toolbar drag and drop
					$('.btn-row', '.btn-toolbar').draggable({
						cursor: 'move',
						helper: 'clone',
						revert: 'invalid',
						connectToSortable: '.rows'
					});
					$('.rows', config).droppable({
						accept: '.btn-row',
						drop: PyAMS_portal.template.dropRowButton
					});
					// Init slot toolbar drag and drop
					$('.btn-slot', '.btn-toolbar').draggable({
						cursor: 'move',
						helper: 'clone',
						revert: 'invalid',
						connectToSortable: '.slots'
					});
					$('.slots', config).droppable({
						accept: '.btn-slot',
						drop: PyAMS_portal.template.dropSlotButton
					});
					// Init portlets toolbar drag and drop
					$('.btn-portlet', '.btn-toolbar').draggable({
						cursor: 'move',
						helper: 'clone',
						revert: 'invalid',
						connectToSortable: '.portlets'
					});
					$('.portlets', config).droppable({
						accept: '.btn-portlet',
						hoverClass: 'portlets-hover',
						activeClass: 'portlets-active',
						drop: PyAMS_portal.template.dropPortletButton
					});
				}
			},


			/**
			 * Display selector
			 */

			selectDisplay: function() {
				var device = $(this).val();
				MyAMS.ajax.post('get-slots-width.json',
								{device: device},
								function(result) {
									var config = $('#portal_config');
									config.removeClassPrefix('container-');
									if (device) {
										config.addClass('container-' + device);
									}
									$('.slot', config).removeClassPrefix('col-');
									for (var slot_name in result) {
										if (!result.hasOwnProperty(slot_name)) {
											continue;
										}
										var widths = result[slot_name];
										var slot = $('.slot[data-ams-slot-name="' + slot_name + '"]', config);
										if (device) {
											slot.addClass('col-' + widths[device]);
										} else {
											for (var display in widths) {
												if (!widths.hasOwnProperty(display)) {
													continue;
												}
												slot.addClass('col-' + display + '-' + widths[display]);
											}
										}
									}
								});
			},

			/**
			 * Rows management
			 */

			addRow: function() {
				return function() {
					$(this).parents('.btn-group').removeClass('open');
					MyAMS.ajax.post('add-template-row.json', {}, function(result) {
						var row_id = result.row_id;
						var rows = $('.rows', '#portal_config');
						$('<div></div>').addClass('row context-menu')
										.attr('data-ams-row-id', row_id)
										.append($('<strong></strong>').addClass('row_id label label-danger pull-left')
																	  .text(row_id+1))
										.append($('<strong></strong>').addClass('row_id label label-danger pull-right')
																	  .text(row_id+1))
										.append($('<div></div>').addClass('slots')
																.sortable({
																	placeholder: 'slot-highlight',
																	connectWith: '.slots',
																	over: PyAMS_portal.template.overSlots,
																	stop: PyAMS_portal.template.sortSlots
																})
																.droppable({
																	accept: '.btn-slot',
																	drop: PyAMS_portal.template.dropSlotButton
																}))
										.contextMenu({
											menuSelector: '#rowMenu',
											menuSelected: MyAMS.helpers.contextMenuHandler
										})
										.appendTo(rows);
						rows.sortable('refresh');
					});
				};
			},

			dropRowButton: function(event, ui) {
				if (ui.draggable.hasClass('already-dropped')) {
					return;
				}
				ui.draggable.addClass('already-dropped');
				MyAMS.ajax.post('add-template-row.json', {}, function(result) {
					var row_id = result.row_id;
					var rows = $('.rows', '#portal_config');
					ui.draggable.removeClassPrefix('btn')
								.removeClassPrefix('ui-')
								.removeClassPrefix('bg-')
								.removeClass('already-dropped')
								.removeAttr('style')
								.addClass('row context-menu')
								.attr('data-ams-row-id', row_id)
								.empty()
								.append($('<strong></strong>').addClass('row_id label label-danger pull-left')
															  .text(row_id+1))
								.append($('<strong></strong>').addClass('row_id label label-danger pull-right')
															  .text(row_id+1))
								.append($('<div></div>').addClass('slots')
														.sortable({
															placeholder: 'slot-highlight',
															connectWith: '.slots',
															over: PyAMS_portal.template.overSlots,
															stop: PyAMS_portal.template.sortSlots
														})
														.droppable({
															accept: '.btn-slot',
															drop: PyAMS_portal.template.dropSlotButton
														}))
								.contextMenu({
									menuSelector: '#rowMenu',
									menuSelected: MyAMS.helpers.contextMenuHandler
								});
					PyAMS_portal.template.sortRows();
					rows.sortable('refresh');
				});
			},

			overRows: function(event, ui) {
				$(ui.placeholder).attr('class', $(ui.item).attr('class'))
								 .removeClassPrefix('ui-')
								 .addClass('row-highlight')
								 .css('height', $(ui.item).outerHeight());
			},

			sortRows: function(event, ui) {
				if (ui && ui.item.hasClass('already-dropped')) {
					return;
				}
				var config = $('#portal_config');
				var ids = $('.row', config).listattr('data-ams-row-id');
				MyAMS.ajax.post('set-template-row-order.json',
								{rows: JSON.stringify(ids)},
								function(result) {
									if (result.status === 'success') {
										$('.row', config).each(function (index) {
											$(this).attr('data-ams-row-id', index);
											$('.row_id', $(this)).text(index+1);
										});
									}
								});
			},

			deleteRow: function() {
				return function(row) {
					row = row.objectOrParentWithClass('row');
					var portlets = $('.portlet', row);
					if (portlets.exists()) {
						MyAMS.skin.messageBox('error', {
							title: MyAMS.i18n.ERROR_OCCURED,
							content: PyAMS_portal.i18n.CANT_DELETE_ROW_WITH_PORTLETS,
							icon: 'fa fa-warning animated shake',
							timeout: 5000
						});
					} else {
						MyAMS.skin.bigBox({
							title: MyAMS.i18n.WARNING,
							content: '<i class="text-danger fa fa-2x fa-bell"></i>&nbsp; ' + MyAMS.i18n.DELETE_WARNING,
							status: 'info',
							buttons: MyAMS.i18n.BTN_OK_CANCEL
						}, function (button) {
							if (button === MyAMS.i18n.BTN_OK) {
								MyAMS.ajax.post('delete-template-row.json',
									{row_id: row.data('ams-row-id')},
									function (result) {
										if (result.status === 'success') {
											row.remove();
											$('.row', '#portal_config').each(function (index) {
												$(this).removeData()
													.attr('data-ams-row-id', index);
												$('.row_id', $(this)).text(index + 1);
											});
										}
									});
							}
						});
					}
				};
			},


			/**
			 * Slots management
			 */

			addSlotCallback: function(result) {
				var slots = $('.slots', '.row[data-ams-row-id="' + result.row_id + '"]');
				var slot_name = result.slot_name;
				var new_slot = $('<div></div>').addClass('slot context-menu no-padding col col-xs-12 col-sm-12 col-md-12 col-lg-12 resizable')
											   .attr('data-ams-slot-name', slot_name)
											   .append($('<div></div>').addClass('header padding-x-5')
																	   .append('<i class="fa fa-fw fa-minus-square pull-right padding-top-2" ' +
																			   '   data-ams-click-handler="PyAMS_portal.template.switchSlot"></i>')
																	   .append(slot_name))
											   .append($('<div></div>').addClass('portlets')
																	   .sortable({
																			placeholder: 'portlet-highlight',
																			connectWith: '.portlets',
																			over: PyAMS_portal.template.overPortlets,
																			stop: PyAMS_portal.template.sortPortlets
																	   })
																	   .droppable({
																			accept: '.btn-portlet',
																			hoverClass: 'portlets-hover',
																			activeClass: 'portlets-active',
																			drop: PyAMS_portal.template.dropPortletButton
																	   }))
											   .append($('<div></div>').addClass('clearfix'))
											   .contextMenu({
													menuSelector: '#slotMenu',
													menuSelected: MyAMS.helpers.contextMenuHandler
											   });
				var slot_button = $('.btn-slot', slots);
				if (slot_button.exists()) {  // Slot added via drag & drop
					slot_button.replaceWith(new_slot);
					$('.slot', slots).each(function() {
						$(this).removeData();
					});
					PyAMS_portal.template.sortSlots();
				} else {
					new_slot.appendTo(slots);
				}
				new_slot.resizable({
					start: PyAMS_portal.template.startSlotResize,
					stop: PyAMS_portal.template.stopSlotResize,
					handles: 'e'
				});
				slots.sortable('refresh');
			},

			dropSlotButton: function(event, ui) {
				if (ui.draggable.hasClass('already-dropped')) {
					return;
				}
				ui.draggable.addClass('already-dropped');
				var row_id = ui.helper.parents('.row:first').data('ams-row-id');
				MyAMS.dialog.open('add-template-slot.html?add_form.widgets.row_id=' + row_id, {}, {
					hide: function() {
						$('.already-dropped').remove();
					}
				});
			},

			startSlotResize: function(event, ui) {
				var slot = ui.element;
				var row = slot.parents('.slots:first');
				var colWidth = (row.innerWidth() - 110) / 12;
				var slotHeight = slot.height();
				ui.element.resizable('option', 'grid', [colWidth, slotHeight]);
				ui.element.resizable('option', 'minWidth', colWidth);
				ui.element.resizable('option', 'minHeight', slotHeight);
				ui.element.resizable('option', 'maxWidth', row.innerWidth());
				ui.element.resizable('option', 'maxHeight', slotHeight);
			},

			stopSlotResize: function(event, ui) {
				var slot = ui.element;
				var row = slot.parents('.slots:first');
				var colWidth = (row.innerWidth() - 10) / 12;
				var slotCols = Math.round($(slot).width() / colWidth);
				var device = $('#device_selector').val();
				if (!device) {
					var deviceWidth = $('body').width();
					if (deviceWidth > 1170) {
						device = 'lg';
					} else if (deviceWidth > 970) {
						device = 'md';
					} else if (deviceWidth > 750) {
						device = 'sm';
					} else {
						device = 'xs';
					}
				}
				MyAMS.ajax.post('set-slot-width.json',
								{slot_name: slot.data('ams-slot-name'),
								 device: device,
								 width: slotCols},
								function(result) {
									slot.removeClassPrefix('col-');
									slot.removeAttr('style');
									var slot_name = slot.data('ams-slot-name');
									var widths = result[slot_name];
									if (device) {
										slot.addClass('col-' + device + '-' + widths[device]);
									} else {
										slot.addClass('col-' + widths[device]);
									}
								});
			},

			editSlot: function() {
				return function(slot) {
					if (!slot.hasClass('slot')) {
						slot = slot.parents('.slot');
					}
					MyAMS.dialog.open('slot-properties.html?slot.widgets.slot_name=' + slot.data('ams-slot-name'));
				};
			},

			editSlotCallback: function(result) {
				var slot = $('.slot[data-ams-slot-name="' + result.slot_name + '"]');
				slot.attr('class', 'slot context-menu no-padding col');
				var device = $('#device_selector').val();
				if (device) {
					slot.addClass('col-' + result.width[device]);
				} else {
					for (device in result.width) {
						if (!result.width.hasOwnProperty(device)) {
							continue;
						}
						slot.addClass('col-' + device + '-' + result.width[device]);
					}
				}
			},

			overSlots: function(event, ui) {
				$(ui.placeholder).attr('class', $(ui.item).attr('class'))
								 .removeClassPrefix('ui-')
								 .addClass('slot-highlight')
								 .css('height', $(ui.item).outerHeight());
			},

			sortSlots: function(event, ui) {
				if (ui && ui.item.hasClass('already-dropped')) {
					return;
				}
				var config = $('#portal_config');
				var order = {};
				$('.row', config).each(function() {
					var row = $(this);
					var row_config = [];
					$('.slot', row).each(function() {
						row_config.push($(this).data('ams-slot-name'));
					});
					order[parseInt(row.attr('data-ams-row-id'))] = row_config;
				});
				MyAMS.ajax.post('set-template-slot-order.json',
								{order: JSON.stringify(order)});
			},

			switchSlot: function() {
				var switcher = $(this);
				var portlets = switcher.parents('.header').first().siblings('.portlets');
				if (portlets.hasClass('hidden')) {
					portlets.removeClass('hidden');
					switcher.removeClass('fa-plus-square')
							.addClass('fa-minus-square');
				} else {
					portlets.addClass('hidden');
					switcher.removeClass('fa-minus-square')
							.addClass('fa-plus-square');
				}
			},

			deleteSlot: function() {
				return function(slot) {
					slot = slot.objectOrParentWithClass('slot');
					var portlets = $('.portlet', slot);
					if (portlets.exists()) {
						MyAMS.skin.messageBox('error', {
							title: MyAMS.i18n.ERROR_OCCURED,
							content: PyAMS_portal.i18n.CANT_DELETE_SLOT_WITH_PORTLETS,
							icon: 'fa fa-warning animated shake',
							timeout: 5000
						});
					} else {
						MyAMS.skin.bigBox({
							title: MyAMS.i18n.WARNING,
							content: '<i class="text-danger fa fa-2x fa-bell"></i>&nbsp; ' + MyAMS.i18n.DELETE_WARNING,
							status: 'info',
							buttons: MyAMS.i18n.BTN_OK_CANCEL
						}, function (button) {
							if (button === MyAMS.i18n.BTN_OK) {
								MyAMS.ajax.post('delete-template-slot.json',
									{slot_name: slot.data('ams-slot-name')},
									function (result) {
										if (result.status === 'success') {
											slot.remove();
											$('.slot', '#portal_config').each(function () {
												$(this).removeData();
											});
										}
									});
							}
						});
					}
				};
			},


			/**
			 * Portlets management
			 */

			addPortletCallback: function(result) {
				var portlets = $('.portlets', '.slot[data-ams-slot-name="' + result.slot_name + '"]');
				var portlet = $('<div></div>').addClass('portlet context-menu')
											  .attr('data-ams-portlet-id', result.portlet_id)
											  .append($('<div></div>').addClass('header padding-x-5')
																	  .append('<i class="fa fa-fw fa-minus-square pull-right padding-top-2" ' +
																			  '   data-ams-click-handler="PyAMS_portal.template.switchPortlet"></i>')
																	  .append(result.label)
																	  .append(' ')
																	  .append('<i class="fa fa-fw fa-edit hint opaque align-base" ' +
																			  '   data-ams-url="PyAMS_portal.template.editPortlet" ' +
																			  '   data-ams-hint-gravity="se" data-ams-hint-offset="2"></i>'))
											  .append($('<div></div>').addClass('preview')
																	  .html(result.preview || ''))
											  .contextMenu({
													menuSelector: '#portletMenu',
													menuSelected: MyAMS.helpers.contextMenuHandler
											  });
				MyAMS.initContent($('.preview', portlet));
				var portlet_button = $('.btn-portlet', portlets);
				if (portlet_button.exists()) {  // Portlet added via drag & drop
					portlet_button.replaceWith(portlet);
					$('.portlet', portlets).each(function() {
						$(this).removeData();
					});
					PyAMS_portal.template.sortPortlets(null, {item: portlet});
				} else {
					portlet.appendTo(portlets);
				}
				portlets.sortable('refresh');
				MyAMS.dialog.open('portlet-properties.html?portlet.widgets.portlet_id=' + result.portlet_id);
			},

			dropPortletButton: function(event, ui) {
				if (ui.draggable.hasClass('already-dropped')) {
					return;
				}
				ui.draggable.addClass('already-dropped');
				var source = ui.draggable;
				var target = $(this);
				var slot = target.parents('.slot:first');
				MyAMS.ajax.post('drag-template-portlet.json', {
					portlet_name: source.data('ams-portlet-name'),
					slot_name: slot.data('ams-slot-name')
				}, function(result) {
					MyAMS.ajax.handleJSON(result);
				});
			},

			submitPortletEditForm: function() {
				var form = $(this).parents('form').first();
				MyAMS.form.submit(form, {form_data: {'autosubmit': true}});
			},

			editPortlet: function() {
				return function(portlet) {
					if (portlet === undefined) {
						portlet = this;
					}
					if (!portlet.hasClass('portlet')) {
						portlet = portlet.parents('.portlet:first');
					}
					MyAMS.dialog.open('portlet-properties.html?portlet.widgets.portlet_id=' + portlet.data('ams-portlet-id'));
				};
			},

			editPortletCallback: function(result) {
				if (result.preview) {
					var config = $('#portal_config');
					var portlet = $('.portlet[data-ams-portlet-id="' + result.portlet_id + '"]', config);
					var header = $('.header', portlet);
					if (result.inherit_parent) {
						$('i.chain', header).removeClass('fa-chain-broken');
					} else {
						$('i.chain', header).addClass('fa-chain-broken');
					}
					$('.preview', portlet).html(result.preview);
					MyAMS.initContent($('.preview', portlet));
				}
			},

			overPortlets: function(event, ui) {
				$(ui.placeholder).attr('class', $(ui.item).attr('class'))
								 .removeClassPrefix('ui-')
								 .addClass('portlet-highlight')
								 .css('height', $(ui.item).outerHeight());
			},

			sortPortlets: function(event, ui) {
				if (ui.item.hasClass('already-dropped')) {
					return;
				}
				var portlet = ui.item;
				var to_slot = portlet.parents('.slot');
				var to_portlets = $('.portlet', to_slot);
				var order = {from: portlet.data('ams-portlet-id'),
							 to: {slot: to_slot.data('ams-slot-name'),
								  portlet_ids: to_portlets.listattr('data-ams-portlet-id')}};
				MyAMS.ajax.post('set-template-portlet-order.json',
								{order: JSON.stringify(order)});
			},

			switchPortlet: function() {
				var switcher = $(this);
				var portlet = switcher.parents('.header').first().siblings('.preview');
				if (portlet.hasClass('hidden')) {
					portlet.removeClass('hidden');
					switcher.removeClass('fa-plus-square')
							.addClass('fa-minus-square');
				} else {
					portlet.addClass('hidden');
					switcher.removeClass('fa-minus-square')
							.addClass('fa-plus-square');
				}
			},

			deletePortlet: function() {
				return function(portlet) {
					MyAMS.skin.bigBox({
						title: MyAMS.i18n.WARNING,
						content: '<i class="text-danger fa fa-2x fa-bell"></i>&nbsp; ' + MyAMS.i18n.DELETE_WARNING,
						status: 'info',
						buttons: MyAMS.i18n.BTN_OK_CANCEL
					}, function(button) {
						if (button === MyAMS.i18n.BTN_OK) {
							if (!portlet.hasClass('portlet')) {
								portlet = portlet.parents('.portlet');
							}
							MyAMS.ajax.post('delete-template-portlet.json',
											{portlet_id: portlet.data('ams-portlet-id')},
											function(result) {
												if (result.status === 'success') {
													portlet.remove();
													$('.portlet', '#portal_config').each(function() {
														$(this).removeData();
													});
												}
											});
						}
					});
				};
			}
		}
	};
	globals.PyAMS_portal = PyAMS_portal;

	var html = $('HTML');
	if (MyAMS.lang !== 'en') {
		MyAMS.getScript('/--static--/pyams_portal/js/i18n/' + MyAMS.lang + MyAMS.devext + '.js');
	}

})(jQuery, this);
